#ifndef MPP_MPP_H_
#define MPP_MPP_H_

#include <list>

#include <FL/fl_ask.H>

#ifdef POSIX__
#include <pthread.h>
//#include <asm/msr.h>
#include <sys/unistd.h>
#include <sys/resource.h>
#define rdtscll(val) __asm__ __volatile__("rdtsc" :"=A"(val))
// Type of val --- unsigned long long 
#endif

#ifdef NTTHREADS__
#include <windows.h>
#include <winbase.h>
#endif

#include <FL/Fl.H>

namespace mpp{
  typedef double Float;	
  class Group;   
  class Widget{
  public:
    virtual void update(){need_update=false;};
    virtual void ask_update(){need_update=true;};
    virtual void set_value(){};		
    Widget();        
    virtual ~Widget(){};		
  protected:
    Group * mpp_parent_;
    bool need_update;
  };

  class Group: public Widget{
  public:
    static Group * current_;
    Group();
    virtual ~Group(){};
    void add(Widget *);
    void mpp_begin();
    void mpp_end();
    void update();
    void ask_update();
    void set_value();        
  private:
    std::list<Widget *> widget_list;
  };   

  class Window: public Group{
  public:
    Window();
    virtual ~Window(){};		
    virtual void show(){};
    virtual void hide(){};		
    virtual const char *label(){return 0;};
  private:
  };
    
  std::list<Widget*>& group_list();		// for update, ...
  std::list<Window*>& window_list();		// for mpp manager

  void update();
  void ask_update();
  void set_value();
  void start();
  void stop();
  inline bool may_i_repeat();
  inline void ask_suspend();
  inline void ask_continue();
  int run(); 
  inline void lock();
  inline void unlock();
  inline void msleep(double t);
#ifdef POSIX__
  extern pthread_t tid;
#endif
#ifdef NTTHREADS__ 
  extern HANDLE thread;
  extern DWORD tid;
#endif
  extern bool ask_stop;    
  extern volatile bool suspend_flag;	
  extern bool calc;	
  extern void (*Calc_func)(void);		// new thread    
  extern void (*Calc_func_)(void);		// same thread
  extern void (*Calc_func_3)(void);		// same thread
  void set_work(void (*f)(void));
  void set_work2(void (*f)(void));
  void set_work3(void (*f)(void), double sleep_time);    // same thread via timer
  extern double sleep_time;
  // task priorities
  extern int priority_hour; 
  extern int priority_day;  
  extern int priority_week; 
  inline void ask_set_priority(int);
  extern int priority_value;      // private
  extern bool priority_changed;   // private
  inline void set_priority(int);       // private 
  

  extern double update_time;	
  inline void set_update_time(double t){update_time=t;};
	
  extern bool visible_v;
  inline bool visible(){return visible_v;}
  inline bool set_visible(bool v){return visible_v=v;}
	
  extern const char * mppg_info;
  inline void mppg_about(Fl_Widget * bt, void *){fl_message(mppg_info);}
	
  // implementation
#ifdef LINUX
#include <sys/resource.h>
  void set_priority(int prio){
    setpriority(PRIO_PROCESS,PRIO_PROCESS,prio);
  }
#else 
  void set_priority(int){}
#endif

  void ask_set_priority(int prior){
    mpp::priority_value=prior;
    mpp::priority_changed=true;
}

#include <FL/Fl.H>
  void lock(){Fl::lock();}
  void unlock(){Fl::unlock();}
  bool may_i_repeat(){
    while(suspend_flag);
    if (priority_changed){
      set_priority(priority_value);
      priority_changed=false;
    }
    return !ask_stop;
  };	
  void ask_suspend(){suspend_flag=true;}
  void ask_continue(){suspend_flag=false;}

  void msleep(double t){
#ifdef POSIX__
    unsigned long long t1;
    unsigned long long t2;
    rdtscll(t1);
    t1+=(unsigned long long)(t*0.7e9);
    do{
      usleep(50);
      rdtscll(t2);}
    while (t1>t2);			
#endif 
#ifdef NTTHREADS__
    SleepEx(DWORD(t*1e+3), false);
#endif 
  }
};

#endif
