#ifndef MPP_TABLE_H
#define MPP_TABLE_H

#include <sstream>

#include <FL/Fl_Window.H>
#include <FL/Fl_Input.H>
#include <FL/Fl_Group.H>

#include <mpp/mpp.h>
  
namespace mpp{
    template <class T> class Input: public Widget, public Fl_Input{
    public:
        Input(T * var, int x, int y, int w, int h, const char *name);
        T* p;
        void update();
        void set_value();
		int handle(int event );
    };

    class Table: public Window, public Fl_Window{
    public:
        Table(int x, int y, int w, int h, const char * name=0); // widget
        Table(int w=250, int h=300, const char * name="Table"); // window
        Table(const char * name);                               // window
		void show(){Fl_Window::show();}
		void hide(){Fl_Window::hide();}
		const char * label(){return Fl_Window::label();};
		int handle(int event);
        template<class T> void add_item(const char * name, T* var, int deact=0){
			//begin(); 
			//mpp_begin();
			Input<T> *tmp=new Input<T>(var, 100, y1, 140, 30, name);
			y1+=35;
			if (deact) tmp->deactivate();
			Fl_Window::add(tmp);
			Group::add(tmp);
			//end(); 
			//mpp_end();
        }
        template<class T> void add_item(const char * name, T& var, int deact=0){
        	add_item(name, &var, deact);
    	}
    private:
        int y1;
    };

    // Implementation
    template<class T> 
    Input<T>::Input(T * var, int x, int y, int w, int h, const char *name): Fl_Input(x, y, w, h, name){
		p=var;
		Widget::ask_update();
		Input::update();
	}
    template<class T> 
    void Input<T>::update(){
		if (need_update){
			Widget::update();
			std::stringstream s;
			s << *p;
			value((s.str()).c_str());
		}		
    }
    template<class T> 
    void Input<T>::set_value(){
        std::string str(value());
        std::stringstream s(str);
        s >> *p;
    }
};

template<class T> 
int mpp::Input<T>::handle(int event ){
	switch(event) {    
		case FL_PUSH:      
			mpp::stop();
		case FL_KEYBOARD:      
			if (Fl::event_key() == FL_Enter) mpp::start();			
	}
	return Fl_Input::handle(event);	
}

#endif
